# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

# NOTE! THIS FILE IS AUTOMATICALLY GENERATED AND WILL BE OVERWRITTEN!
#
# IF YOU WANT TO MODIFY THIS FILE, YOU SHOULD MODIFY THE TEMPLATE
# `get_provider_info_TEMPLATE.py.jinja2` IN the `dev/breeze/src/airflow_breeze/templates` DIRECTORY


def get_provider_info():
    return {
        "package-name": "apache-airflow-providers-amazon",
        "name": "Amazon",
        "description": "Amazon integration (including `Amazon Web Services (AWS) <https://aws.amazon.com/>`__).\n",
        "integrations": [
            {
                "integration-name": "Amazon Athena",
                "external-doc-url": "https://aws.amazon.com/athena/",
                "logo": "/docs/integration-logos/Amazon-Athena_light-bg@4x.png",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-amazon/operators/athena/athena_boto.rst",
                    "/docs/apache-airflow-providers-amazon/operators/athena/athena_sql.rst",
                ],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Bedrock",
                "external-doc-url": "https://aws.amazon.com/bedrock/",
                "logo": "/docs/integration-logos/Amazon-Bedrock_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/bedrock.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Chime",
                "external-doc-url": "https://aws.amazon.com/chime/",
                "logo": "/docs/integration-logos/Amazon-Chime-light-bg.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon CloudFormation",
                "external-doc-url": "https://aws.amazon.com/cloudformation/",
                "logo": "/docs/integration-logos/AWS-CloudFormation_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/cloudformation.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon CloudWatch Logs",
                "external-doc-url": "https://aws.amazon.com/cloudwatch/",
                "logo": "/docs/integration-logos/Amazon-CloudWatch_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Comprehend",
                "external-doc-url": "https://aws.amazon.com/comprehend/",
                "logo": "/docs/integration-logos/Amazon-Comprehend_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/comprehend.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon DataSync",
                "external-doc-url": "https://aws.amazon.com/datasync/",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/datasync.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon DynamoDB",
                "external-doc-url": "https://aws.amazon.com/dynamodb/",
                "logo": "/docs/integration-logos/Amazon-DynamoDB_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/dynamodb.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon EC2",
                "external-doc-url": "https://aws.amazon.com/ec2/",
                "logo": "/docs/integration-logos/Amazon-EC2_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/ec2.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Elastic Container Registry (ECR)",
                "external-doc-url": "https://aws.amazon.com/ecr/",
                "logo": "/docs/integration-logos/Amazon-Elastic-Container-Registry_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon ECS",
                "external-doc-url": "https://aws.amazon.com/ecs/",
                "logo": "/docs/integration-logos/Amazon-Elastic-Container-Service_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Elastic Kubernetes Service (EKS)",
                "external-doc-url": "https://aws.amazon.com/eks/",
                "logo": "/docs/integration-logos/Amazon-Elastic-Kubernetes-Service_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/eks.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon ElastiCache",
                "external-doc-url": "https://aws.amazon.com/elasticache/redis//",
                "logo": "/docs/integration-logos/Amazon-ElastiCache_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon EMR",
                "external-doc-url": "https://aws.amazon.com/emr/",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/emr/emr.rst"],
                "logo": "/docs/integration-logos/Amazon-EMR_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon EMR on EKS",
                "external-doc-url": "https://docs.aws.amazon.com/emr/latest/EMR-on-EKS-DevelopmentGuide/emr-eks.html",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/emr/emr_eks.rst"],
                "logo": "/docs/integration-logos/Amazon-EMR_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon EMR Serverless",
                "external-doc-url": "https://docs.aws.amazon.com/emr/latest/EMR-Serverless-UserGuide/emr-serverless.html",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/emr/emr_serverless.rst"],
                "logo": "/docs/integration-logos/Amazon-EMR_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon EventBridge",
                "external-doc-url": "https://docs.aws.amazon.com/eventbridge/latest/APIReference/Welcome.html",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/eventbridge.rst"],
                "logo": "/docs/integration-logos/Amazon-EventBridge_64.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Glacier",
                "external-doc-url": "https://aws.amazon.com/glacier/",
                "logo": "/docs/integration-logos/Amazon-S3-Glacier_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/s3/glacier.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Kinesis Data Firehose",
                "external-doc-url": "https://aws.amazon.com/kinesis/data-firehose/",
                "logo": "/docs/integration-logos/Amazon-Kinesis-Data-Firehose_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Managed Service for Apache Flink",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/kinesis_analytics.rst"],
                "external-doc-url": "https://aws.amazon.com/managed-service-apache-flink/",
                "logo": "/docs/integration-logos/Amazon-Kinesis-Analytics_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon OpenSearch Serverless",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/opensearchserverless.rst"],
                "external-doc-url": "https://aws.amazon.com/opensearchserverless/",
                "logo": "/docs/integration-logos/Amazon-OpenSearch_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon RDS",
                "external-doc-url": "https://aws.amazon.com/rds/",
                "logo": "/docs/integration-logos/Amazon-RDS_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/rds.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Redshift",
                "external-doc-url": "https://aws.amazon.com/redshift/",
                "logo": "/docs/integration-logos/Amazon-Redshift_light-bg@4x.png",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-amazon/operators/redshift/redshift_sql.rst",
                    "/docs/apache-airflow-providers-amazon/operators/redshift/redshift_cluster.rst",
                ],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Redshift Data",
                "external-doc-url": "https://aws.amazon.com/redshift/",
                "logo": "/docs/integration-logos/Amazon-Redshift_light-bg@4x.png",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-amazon/operators/redshift/redshift_data.rst"
                ],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon SageMaker",
                "external-doc-url": "https://aws.amazon.com/sagemaker/",
                "logo": "/docs/integration-logos/Amazon-SageMaker_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/sagemaker.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon SageMaker Unified Studio",
                "external-doc-url": "https://aws.amazon.com/sagemaker/unified-studio/",
                "logo": "/docs/integration-logos/Amazon-SageMaker_light-bg@4x.png",
                "how-to-guide": [
                    "/docs/apache-airflow-providers-amazon/operators/sagemakerunifiedstudio.rst"
                ],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon SecretsManager",
                "external-doc-url": "https://aws.amazon.com/secrets-manager/",
                "logo": "/docs/integration-logos/AWS-Secrets-Manager_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Simple Email Service (SES)",
                "external-doc-url": "https://aws.amazon.com/ses/",
                "logo": "/docs/integration-logos/Amazon-Simple-Email-Service-SES_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/ecs.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Simple Notification Service (SNS)",
                "external-doc-url": "https://aws.amazon.com/sns/",
                "logo": "/docs/integration-logos/Amazon-Simple-Notification-Service-SNS_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/sns.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Simple Queue Service (SQS)",
                "external-doc-url": "https://aws.amazon.com/sqs/",
                "logo": "/docs/integration-logos/Amazon-Simple-Queue-Service-SQS_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/sqs.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Simple Storage Service (S3)",
                "external-doc-url": "https://aws.amazon.com/s3/",
                "logo": "/docs/integration-logos/Amazon-Simple-Storage-Service-S3_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/s3/s3.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Systems Manager (SSM)",
                "external-doc-url": "https://aws.amazon.com/systems-manager/",
                "logo": "/docs/integration-logos/AWS-Systems-Manager_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/ssm.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Web Services",
                "external-doc-url": "https://aws.amazon.com/",
                "logo": "/docs/integration-logos/AWS-Cloud-alt_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "AWS Batch",
                "external-doc-url": "https://aws.amazon.com/batch/",
                "logo": "/docs/integration-logos/AWS-Batch_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/batch.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "AWS DataSync",
                "external-doc-url": "https://aws.amazon.com/datasync/",
                "logo": "/docs/integration-logos/AWS-DataSync_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "AWS Glue",
                "external-doc-url": "https://aws.amazon.com/glue/",
                "logo": "/docs/integration-logos/AWS-Glue_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/glue.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "AWS Lambda",
                "external-doc-url": "https://aws.amazon.com/lambda/",
                "logo": "/docs/integration-logos/AWS-Lambda_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/lambda.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "AWS Step Functions",
                "external-doc-url": "https://aws.amazon.com/step-functions/",
                "logo": "/docs/integration-logos/AWS-Step-Functions_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/step_functions.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "AWS Database Migration Service",
                "external-doc-url": "https://aws.amazon.com/dms/",
                "logo": "/docs/integration-logos/AWS-Database-Migration-Service_64@5x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/dms.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon QuickSight",
                "external-doc-url": "https://aws.amazon.com/quicksight/",
                "logo": "/docs/integration-logos/Amazon-Quicksight_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/quicksight.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "AWS Security Token Service (STS)",
                "external-doc-url": "https://docs.aws.amazon.com/STS/latest/APIReference/welcome.html",
                "logo": "/docs/integration-logos/AWS-STS_light-bg@4x.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Appflow",
                "external-doc-url": "https://docs.aws.amazon.com/appflow/1.0/APIReference/Welcome.html",
                "logo": "/docs/integration-logos/Amazon_AppFlow_light.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/appflow.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "AWS Glue DataBrew",
                "external-doc-url": "https://docs.aws.amazon.com/databrew/latest/dg/what-is.html",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/glue_databrew.rst"],
                "logo": "/docs/integration-logos/AWS-Glue-DataBrew_64.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Verified Permissions",
                "external-doc-url": "https://aws.amazon.com/verified-permissions/",
                "logo": "/docs/integration-logos/Amazon-Verified-Permissions.png",
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Neptune",
                "external-doc-url": "https://aws.amazon.com/neptune/",
                "logo": "/docs/integration-logos/Amazon-Neptune_64.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/neptune.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Kinesis Data Stream",
                "external-doc-url": "https://aws.amazon.com/kinesis/",
                "logo": "/docs/integration-logos/Amazon-Kinesis-Data-Firehose_light-bg@4x.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/kinesis_analytics.rst"],
                "tags": ["aws"],
            },
            {
                "integration-name": "Amazon Managed Workflows for Apache Airflow (MWAA)",
                "external-doc-url": "https://aws.amazon.com/managed-workflows-for-apache-airflow/",
                "logo": "/docs/integration-logos/Amazon-MWAA.png",
                "how-to-guide": ["/docs/apache-airflow-providers-amazon/operators/mwaa.rst"],
                "tags": ["aws"],
            },
        ],
        "operators": [
            {
                "integration-name": "Amazon Athena",
                "python-modules": ["airflow.providers.amazon.aws.operators.athena"],
            },
            {
                "integration-name": "Amazon Web Services",
                "python-modules": ["airflow.providers.amazon.aws.operators.base_aws"],
            },
            {
                "integration-name": "AWS Batch",
                "python-modules": ["airflow.providers.amazon.aws.operators.batch"],
            },
            {
                "integration-name": "Amazon Bedrock",
                "python-modules": ["airflow.providers.amazon.aws.operators.bedrock"],
            },
            {
                "integration-name": "Amazon CloudFormation",
                "python-modules": ["airflow.providers.amazon.aws.operators.cloud_formation"],
            },
            {
                "integration-name": "Amazon Comprehend",
                "python-modules": ["airflow.providers.amazon.aws.operators.comprehend"],
            },
            {
                "integration-name": "Amazon DataSync",
                "python-modules": ["airflow.providers.amazon.aws.operators.datasync"],
            },
            {
                "integration-name": "AWS Database Migration Service",
                "python-modules": ["airflow.providers.amazon.aws.operators.dms"],
            },
            {
                "integration-name": "Amazon EC2",
                "python-modules": ["airflow.providers.amazon.aws.operators.ec2"],
            },
            {
                "integration-name": "Amazon ECS",
                "python-modules": ["airflow.providers.amazon.aws.operators.ecs"],
            },
            {
                "integration-name": "Amazon Elastic Kubernetes Service (EKS)",
                "python-modules": ["airflow.providers.amazon.aws.operators.eks"],
            },
            {
                "integration-name": "Amazon EMR",
                "python-modules": ["airflow.providers.amazon.aws.operators.emr"],
            },
            {
                "integration-name": "Amazon EMR on EKS",
                "python-modules": ["airflow.providers.amazon.aws.operators.emr"],
            },
            {
                "integration-name": "Amazon EventBridge",
                "python-modules": ["airflow.providers.amazon.aws.operators.eventbridge"],
            },
            {
                "integration-name": "Amazon Glacier",
                "python-modules": ["airflow.providers.amazon.aws.operators.glacier"],
            },
            {
                "integration-name": "AWS Glue",
                "python-modules": [
                    "airflow.providers.amazon.aws.operators.glue",
                    "airflow.providers.amazon.aws.operators.glue_crawler",
                ],
            },
            {
                "integration-name": "AWS Lambda",
                "python-modules": ["airflow.providers.amazon.aws.operators.lambda_function"],
            },
            {
                "integration-name": "Amazon Managed Service for Apache Flink",
                "python-modules": ["airflow.providers.amazon.aws.operators.kinesis_analytics"],
            },
            {
                "integration-name": "Amazon Managed Workflows for Apache Airflow (MWAA)",
                "python-modules": ["airflow.providers.amazon.aws.operators.mwaa"],
            },
            {
                "integration-name": "Amazon Simple Storage Service (S3)",
                "python-modules": ["airflow.providers.amazon.aws.operators.s3"],
            },
            {
                "integration-name": "Amazon SageMaker",
                "python-modules": ["airflow.providers.amazon.aws.operators.sagemaker"],
            },
            {
                "integration-name": "Amazon SageMaker Unified Studio",
                "python-modules": ["airflow.providers.amazon.aws.operators.sagemaker_unified_studio"],
            },
            {
                "integration-name": "Amazon Simple Notification Service (SNS)",
                "python-modules": ["airflow.providers.amazon.aws.operators.sns"],
            },
            {
                "integration-name": "Amazon Simple Queue Service (SQS)",
                "python-modules": ["airflow.providers.amazon.aws.operators.sqs"],
            },
            {
                "integration-name": "AWS Step Functions",
                "python-modules": ["airflow.providers.amazon.aws.operators.step_function"],
            },
            {
                "integration-name": "Amazon Systems Manager (SSM)",
                "python-modules": ["airflow.providers.amazon.aws.operators.ssm"],
            },
            {
                "integration-name": "Amazon RDS",
                "python-modules": ["airflow.providers.amazon.aws.operators.rds"],
            },
            {
                "integration-name": "Amazon Redshift",
                "python-modules": [
                    "airflow.providers.amazon.aws.operators.redshift_cluster",
                    "airflow.providers.amazon.aws.operators.redshift_data",
                ],
            },
            {
                "integration-name": "Amazon QuickSight",
                "python-modules": ["airflow.providers.amazon.aws.operators.quicksight"],
            },
            {
                "integration-name": "Amazon Appflow",
                "python-modules": ["airflow.providers.amazon.aws.operators.appflow"],
            },
            {
                "integration-name": "AWS Glue DataBrew",
                "python-modules": ["airflow.providers.amazon.aws.operators.glue_databrew"],
            },
            {
                "integration-name": "Amazon Neptune",
                "python-modules": ["airflow.providers.amazon.aws.operators.neptune"],
            },
        ],
        "sensors": [
            {
                "integration-name": "Amazon Athena",
                "python-modules": ["airflow.providers.amazon.aws.sensors.athena"],
            },
            {
                "integration-name": "Amazon Web Services",
                "python-modules": ["airflow.providers.amazon.aws.sensors.base_aws"],
            },
            {
                "integration-name": "AWS Batch",
                "python-modules": ["airflow.providers.amazon.aws.sensors.batch"],
            },
            {
                "integration-name": "Amazon Bedrock",
                "python-modules": ["airflow.providers.amazon.aws.sensors.bedrock"],
            },
            {
                "integration-name": "Amazon CloudFormation",
                "python-modules": ["airflow.providers.amazon.aws.sensors.cloud_formation"],
            },
            {
                "integration-name": "Amazon Comprehend",
                "python-modules": ["airflow.providers.amazon.aws.sensors.comprehend"],
            },
            {
                "integration-name": "AWS Database Migration Service",
                "python-modules": ["airflow.providers.amazon.aws.sensors.dms"],
            },
            {
                "integration-name": "Amazon DynamoDB",
                "python-modules": ["airflow.providers.amazon.aws.sensors.dynamodb"],
            },
            {
                "integration-name": "Amazon EC2",
                "python-modules": ["airflow.providers.amazon.aws.sensors.ec2"],
            },
            {
                "integration-name": "Amazon ECS",
                "python-modules": ["airflow.providers.amazon.aws.sensors.ecs"],
            },
            {
                "integration-name": "Amazon Elastic Kubernetes Service (EKS)",
                "python-modules": ["airflow.providers.amazon.aws.sensors.eks"],
            },
            {
                "integration-name": "Amazon EMR",
                "python-modules": ["airflow.providers.amazon.aws.sensors.emr"],
            },
            {
                "integration-name": "Amazon EMR on EKS",
                "python-modules": ["airflow.providers.amazon.aws.sensors.emr"],
            },
            {
                "integration-name": "Amazon Glacier",
                "python-modules": ["airflow.providers.amazon.aws.sensors.glacier"],
            },
            {
                "integration-name": "AWS Glue",
                "python-modules": [
                    "airflow.providers.amazon.aws.sensors.glue",
                    "airflow.providers.amazon.aws.sensors.glue_crawler",
                    "airflow.providers.amazon.aws.sensors.glue_catalog_partition",
                ],
            },
            {
                "integration-name": "AWS Lambda",
                "python-modules": ["airflow.providers.amazon.aws.sensors.lambda_function"],
            },
            {
                "integration-name": "Amazon Managed Service for Apache Flink",
                "python-modules": ["airflow.providers.amazon.aws.sensors.kinesis_analytics"],
            },
            {
                "integration-name": "Amazon Managed Workflows for Apache Airflow (MWAA)",
                "python-modules": ["airflow.providers.amazon.aws.sensors.mwaa"],
            },
            {
                "integration-name": "Amazon OpenSearch Serverless",
                "python-modules": ["airflow.providers.amazon.aws.sensors.opensearch_serverless"],
            },
            {
                "integration-name": "Amazon RDS",
                "python-modules": ["airflow.providers.amazon.aws.sensors.rds"],
            },
            {
                "integration-name": "Amazon Redshift",
                "python-modules": ["airflow.providers.amazon.aws.sensors.redshift_cluster"],
            },
            {
                "integration-name": "Amazon Simple Storage Service (S3)",
                "python-modules": ["airflow.providers.amazon.aws.sensors.s3"],
            },
            {
                "integration-name": "Amazon SageMaker",
                "python-modules": ["airflow.providers.amazon.aws.sensors.sagemaker"],
            },
            {
                "integration-name": "Amazon SageMaker Unified Studio",
                "python-modules": ["airflow.providers.amazon.aws.sensors.sagemaker_unified_studio"],
            },
            {
                "integration-name": "Amazon Simple Queue Service (SQS)",
                "python-modules": ["airflow.providers.amazon.aws.sensors.sqs"],
            },
            {
                "integration-name": "AWS Step Functions",
                "python-modules": ["airflow.providers.amazon.aws.sensors.step_function"],
            },
            {
                "integration-name": "Amazon Systems Manager (SSM)",
                "python-modules": ["airflow.providers.amazon.aws.sensors.ssm"],
            },
            {
                "integration-name": "Amazon QuickSight",
                "python-modules": ["airflow.providers.amazon.aws.sensors.quicksight"],
            },
        ],
        "asset-uris": [
            {
                "schemes": ["s3"],
                "handler": "airflow.providers.amazon.aws.assets.s3.sanitize_uri",
                "to_openlineage_converter": "airflow.providers.amazon.aws.assets.s3.convert_asset_to_openlineage",
                "factory": "airflow.providers.amazon.aws.assets.s3.create_asset",
            }
        ],
        "dataset-uris": [
            {
                "schemes": ["s3"],
                "handler": "airflow.providers.amazon.aws.assets.s3.sanitize_uri",
                "to_openlineage_converter": "airflow.providers.amazon.aws.assets.s3.convert_asset_to_openlineage",
                "factory": "airflow.providers.amazon.aws.assets.s3.create_asset",
            }
        ],
        "filesystems": ["airflow.providers.amazon.aws.fs.s3"],
        "hooks": [
            {
                "integration-name": "Amazon Athena",
                "python-modules": [
                    "airflow.providers.amazon.aws.hooks.athena",
                    "airflow.providers.amazon.aws.hooks.athena_sql",
                ],
            },
            {
                "integration-name": "Amazon Bedrock",
                "python-modules": ["airflow.providers.amazon.aws.hooks.bedrock"],
            },
            {
                "integration-name": "Amazon Chime",
                "python-modules": ["airflow.providers.amazon.aws.hooks.chime"],
            },
            {
                "integration-name": "Amazon Comprehend",
                "python-modules": ["airflow.providers.amazon.aws.hooks.comprehend"],
            },
            {
                "integration-name": "Amazon DynamoDB",
                "python-modules": ["airflow.providers.amazon.aws.hooks.dynamodb"],
            },
            {
                "integration-name": "Amazon Web Services",
                "python-modules": ["airflow.providers.amazon.aws.hooks.base_aws"],
            },
            {
                "integration-name": "AWS Batch",
                "python-modules": [
                    "airflow.providers.amazon.aws.hooks.batch_client",
                    "airflow.providers.amazon.aws.hooks.batch_waiters",
                ],
            },
            {
                "integration-name": "Amazon CloudFormation",
                "python-modules": ["airflow.providers.amazon.aws.hooks.cloud_formation"],
            },
            {
                "integration-name": "Amazon DataSync",
                "python-modules": ["airflow.providers.amazon.aws.hooks.datasync"],
            },
            {
                "integration-name": "AWS Database Migration Service",
                "python-modules": ["airflow.providers.amazon.aws.hooks.dms"],
            },
            {"integration-name": "Amazon EC2", "python-modules": ["airflow.providers.amazon.aws.hooks.ec2"]},
            {
                "integration-name": "Amazon Elastic Container Registry (ECR)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.ecr"],
            },
            {"integration-name": "Amazon ECS", "python-modules": ["airflow.providers.amazon.aws.hooks.ecs"]},
            {
                "integration-name": "Amazon ElastiCache",
                "python-modules": ["airflow.providers.amazon.aws.hooks.elasticache_replication_group"],
            },
            {
                "integration-name": "Amazon Elastic Kubernetes Service (EKS)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.eks"],
            },
            {"integration-name": "Amazon EMR", "python-modules": ["airflow.providers.amazon.aws.hooks.emr"]},
            {
                "integration-name": "Amazon EMR on EKS",
                "python-modules": ["airflow.providers.amazon.aws.hooks.emr"],
            },
            {
                "integration-name": "Amazon EventBridge",
                "python-modules": ["airflow.providers.amazon.aws.hooks.eventbridge"],
            },
            {
                "integration-name": "Amazon Glacier",
                "python-modules": ["airflow.providers.amazon.aws.hooks.glacier"],
            },
            {
                "integration-name": "AWS Glue",
                "python-modules": [
                    "airflow.providers.amazon.aws.hooks.glue",
                    "airflow.providers.amazon.aws.hooks.glue_crawler",
                    "airflow.providers.amazon.aws.hooks.glue_catalog",
                ],
            },
            {
                "integration-name": "Amazon Kinesis Data Stream",
                "python-modules": ["airflow.providers.amazon.aws.hooks.kinesis"],
            },
            {
                "integration-name": "Amazon Kinesis Data Firehose",
                "python-modules": ["airflow.providers.amazon.aws.hooks.firehose"],
            },
            {
                "integration-name": "AWS Lambda",
                "python-modules": ["airflow.providers.amazon.aws.hooks.lambda_function"],
            },
            {
                "integration-name": "Amazon Managed Service for Apache Flink",
                "python-modules": ["airflow.providers.amazon.aws.hooks.kinesis_analytics"],
            },
            {
                "integration-name": "Amazon CloudWatch Logs",
                "python-modules": ["airflow.providers.amazon.aws.hooks.logs"],
            },
            {
                "integration-name": "Amazon Managed Workflows for Apache Airflow (MWAA)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.mwaa"],
            },
            {
                "integration-name": "Amazon OpenSearch Serverless",
                "python-modules": ["airflow.providers.amazon.aws.hooks.opensearch_serverless"],
            },
            {"integration-name": "Amazon RDS", "python-modules": ["airflow.providers.amazon.aws.hooks.rds"]},
            {
                "integration-name": "Amazon Redshift",
                "python-modules": [
                    "airflow.providers.amazon.aws.hooks.redshift_sql",
                    "airflow.providers.amazon.aws.hooks.redshift_cluster",
                    "airflow.providers.amazon.aws.hooks.redshift_data",
                ],
            },
            {
                "integration-name": "Amazon Simple Storage Service (S3)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.s3"],
            },
            {
                "integration-name": "Amazon SageMaker",
                "python-modules": ["airflow.providers.amazon.aws.hooks.sagemaker"],
            },
            {
                "integration-name": "Amazon SageMaker Unified Studio",
                "python-modules": ["airflow.providers.amazon.aws.hooks.sagemaker_unified_studio"],
            },
            {
                "integration-name": "Amazon Simple Email Service (SES)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.ses"],
            },
            {
                "integration-name": "Amazon Systems Manager (SSM)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.ssm"],
            },
            {
                "integration-name": "Amazon SecretsManager",
                "python-modules": ["airflow.providers.amazon.aws.hooks.secrets_manager"],
            },
            {
                "integration-name": "Amazon Simple Notification Service (SNS)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.sns"],
            },
            {
                "integration-name": "Amazon Simple Queue Service (SQS)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.sqs"],
            },
            {
                "integration-name": "AWS Step Functions",
                "python-modules": ["airflow.providers.amazon.aws.hooks.step_function"],
            },
            {
                "integration-name": "Amazon QuickSight",
                "python-modules": ["airflow.providers.amazon.aws.hooks.quicksight"],
            },
            {
                "integration-name": "AWS Security Token Service (STS)",
                "python-modules": ["airflow.providers.amazon.aws.hooks.sts"],
            },
            {
                "integration-name": "Amazon Appflow",
                "python-modules": ["airflow.providers.amazon.aws.hooks.appflow"],
            },
            {
                "integration-name": "AWS Glue DataBrew",
                "python-modules": ["airflow.providers.amazon.aws.hooks.glue_databrew"],
            },
            {
                "integration-name": "Amazon Verified Permissions",
                "python-modules": ["airflow.providers.amazon.aws.hooks.verified_permissions"],
            },
            {
                "integration-name": "Amazon Neptune",
                "python-modules": ["airflow.providers.amazon.aws.hooks.neptune"],
            },
        ],
        "triggers": [
            {
                "integration-name": "Amazon Web Services",
                "python-modules": ["airflow.providers.amazon.aws.triggers.base"],
            },
            {
                "integration-name": "Amazon Athena",
                "python-modules": ["airflow.providers.amazon.aws.triggers.athena"],
            },
            {
                "integration-name": "AWS Batch",
                "python-modules": ["airflow.providers.amazon.aws.triggers.batch"],
            },
            {
                "integration-name": "Amazon Bedrock",
                "python-modules": ["airflow.providers.amazon.aws.triggers.bedrock"],
            },
            {
                "integration-name": "Amazon Comprehend",
                "python-modules": ["airflow.providers.amazon.aws.triggers.comprehend"],
            },
            {
                "integration-name": "Amazon EC2",
                "python-modules": ["airflow.providers.amazon.aws.triggers.ec2"],
            },
            {
                "integration-name": "AWS Lambda",
                "python-modules": ["airflow.providers.amazon.aws.triggers.lambda_function"],
            },
            {
                "integration-name": "Amazon Managed Workflows for Apache Airflow (MWAA)",
                "python-modules": ["airflow.providers.amazon.aws.triggers.mwaa"],
            },
            {
                "integration-name": "Amazon Managed Service for Apache Flink",
                "python-modules": ["airflow.providers.amazon.aws.triggers.kinesis_analytics"],
            },
            {
                "integration-name": "Amazon OpenSearch Serverless",
                "python-modules": ["airflow.providers.amazon.aws.triggers.opensearch_serverless"],
            },
            {
                "integration-name": "Amazon Redshift",
                "python-modules": [
                    "airflow.providers.amazon.aws.triggers.redshift_cluster",
                    "airflow.providers.amazon.aws.triggers.redshift_data",
                ],
            },
            {
                "integration-name": "Amazon SageMaker",
                "python-modules": ["airflow.providers.amazon.aws.triggers.sagemaker"],
            },
            {
                "integration-name": "Amazon SageMaker Unified Studio",
                "python-modules": ["airflow.providers.amazon.aws.triggers.sagemaker_unified_studio"],
            },
            {
                "integration-name": "AWS Glue",
                "python-modules": [
                    "airflow.providers.amazon.aws.triggers.glue",
                    "airflow.providers.amazon.aws.triggers.glue_crawler",
                ],
            },
            {
                "integration-name": "Amazon Simple Storage Service (S3)",
                "python-modules": ["airflow.providers.amazon.aws.triggers.s3"],
            },
            {
                "integration-name": "Amazon Systems Manager (SSM)",
                "python-modules": ["airflow.providers.amazon.aws.triggers.ssm"],
            },
            {
                "integration-name": "Amazon EMR",
                "python-modules": ["airflow.providers.amazon.aws.triggers.emr"],
            },
            {
                "integration-name": "Amazon Elastic Kubernetes Service (EKS)",
                "python-modules": ["airflow.providers.amazon.aws.triggers.eks"],
            },
            {
                "integration-name": "Amazon ECS",
                "python-modules": ["airflow.providers.amazon.aws.triggers.ecs"],
            },
            {
                "integration-name": "Amazon RDS",
                "python-modules": ["airflow.providers.amazon.aws.triggers.rds"],
            },
            {
                "integration-name": "AWS Step Functions",
                "python-modules": ["airflow.providers.amazon.aws.triggers.step_function"],
            },
            {
                "integration-name": "Amazon Simple Queue Service (SQS)",
                "python-modules": ["airflow.providers.amazon.aws.triggers.sqs"],
            },
            {
                "integration-name": "AWS Glue DataBrew",
                "python-modules": ["airflow.providers.amazon.aws.triggers.glue_databrew"],
            },
            {
                "integration-name": "Amazon Neptune",
                "python-modules": ["airflow.providers.amazon.aws.triggers.neptune"],
            },
            {
                "integration-name": "AWS Database Migration Service",
                "python-modules": ["airflow.providers.amazon.aws.triggers.dms"],
            },
        ],
        "transfers": [
            {
                "source-integration-name": "Amazon DynamoDB",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/dynamodb_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.dynamodb_to_s3",
            },
            {
                "source-integration-name": "Google Cloud Storage (GCS)",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/gcs_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.gcs_to_s3",
            },
            {
                "source-integration-name": "Amazon Glacier",
                "target-integration-name": "Google Cloud Storage (GCS)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/glacier_to_gcs.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.glacier_to_gcs",
            },
            {
                "source-integration-name": "Google",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/google_api_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.google_api_to_s3",
            },
            {
                "source-integration-name": "Apache Hive",
                "target-integration-name": "Amazon DynamoDB",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/hive_to_dynamodb.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.hive_to_dynamodb",
            },
            {
                "source-integration-name": "Hypertext Transfer Protocol (HTTP)",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/http_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.http_to_s3",
            },
            {
                "source-integration-name": "Internet Message Access Protocol (IMAP)",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/imap_attachment_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.imap_attachment_to_s3",
            },
            {
                "source-integration-name": "MongoDB",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/mongo_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.mongo_to_s3",
            },
            {
                "source-integration-name": "Amazon Redshift",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/redshift_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.redshift_to_s3",
            },
            {
                "source-integration-name": "Amazon Simple Storage Service (S3)",
                "target-integration-name": "Amazon Redshift",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/s3_to_redshift.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.s3_to_redshift",
            },
            {
                "source-integration-name": "Amazon Simple Storage Service (S3)",
                "target-integration-name": "SSH File Transfer Protocol (SFTP)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/s3_to_sftp.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.s3_to_sftp",
            },
            {
                "source-integration-name": "SSH File Transfer Protocol (SFTP)",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/sftp_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.sftp_to_s3",
            },
            {
                "source-integration-name": "Amazon Simple Storage Service (S3)",
                "target-integration-name": "File Transfer Protocol (FTP)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/s3_to_ftp.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.s3_to_ftp",
            },
            {
                "source-integration-name": "Exasol",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "python-module": "airflow.providers.amazon.aws.transfers.exasol_to_s3",
            },
            {
                "source-integration-name": "File Transfer Protocol (FTP)",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/ftp_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.ftp_to_s3",
            },
            {
                "source-integration-name": "Salesforce",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/salesforce_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.salesforce_to_s3",
            },
            {
                "source-integration-name": "Local",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/local_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.local_to_s3",
            },
            {
                "source-integration-name": "Common SQL",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/sql_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.sql_to_s3",
            },
            {
                "source-integration-name": "Amazon Simple Storage Service (S3)",
                "target-integration-name": "Common SQL",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/s3_to_sql.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.s3_to_sql",
            },
            {
                "source-integration-name": "Amazon Web Services",
                "target-integration-name": "Amazon Web Services",
                "python-module": "airflow.providers.amazon.aws.transfers.base",
            },
            {
                "source-integration-name": "Microsoft Azure Blob Storage",
                "target-integration-name": "Amazon Simple Storage Service (S3)",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/azure_blob_to_s3.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.azure_blob_to_s3",
            },
            {
                "source-integration-name": "Amazon Simple Storage Service (S3)",
                "target-integration-name": "Amazon DynamoDB",
                "how-to-guide": "/docs/apache-airflow-providers-amazon/transfer/s3_to_dynamodb.rst",
                "python-module": "airflow.providers.amazon.aws.transfers.s3_to_dynamodb",
            },
        ],
        "extra-links": [
            "airflow.providers.amazon.aws.links.athena.AthenaQueryResultsLink",
            "airflow.providers.amazon.aws.links.batch.BatchJobDefinitionLink",
            "airflow.providers.amazon.aws.links.batch.BatchJobDetailsLink",
            "airflow.providers.amazon.aws.links.batch.BatchJobQueueLink",
            "airflow.providers.amazon.aws.links.emr.EmrClusterLink",
            "airflow.providers.amazon.aws.links.emr.EmrLogsLink",
            "airflow.providers.amazon.aws.links.emr.EmrServerlessCloudWatchLogsLink",
            "airflow.providers.amazon.aws.links.emr.EmrServerlessDashboardLink",
            "airflow.providers.amazon.aws.links.emr.EmrServerlessLogsLink",
            "airflow.providers.amazon.aws.links.emr.EmrServerlessS3LogsLink",
            "airflow.providers.amazon.aws.links.glue.GlueJobRunDetailsLink",
            "airflow.providers.amazon.aws.links.logs.CloudWatchEventsLink",
            "airflow.providers.amazon.aws.links.sagemaker.SageMakerTransformJobLink",
            "airflow.providers.amazon.aws.links.sagemaker_unified_studio.SageMakerUnifiedStudioLink",
            "airflow.providers.amazon.aws.links.step_function.StateMachineDetailsLink",
            "airflow.providers.amazon.aws.links.step_function.StateMachineExecutionsDetailsLink",
            "airflow.providers.amazon.aws.links.comprehend.ComprehendPiiEntitiesDetectionLink",
            "airflow.providers.amazon.aws.links.comprehend.ComprehendDocumentClassifierLink",
            "airflow.providers.amazon.aws.links.datasync.DataSyncTaskLink",
            "airflow.providers.amazon.aws.links.datasync.DataSyncTaskExecutionLink",
            "airflow.providers.amazon.aws.links.ec2.EC2InstanceLink",
            "airflow.providers.amazon.aws.links.ec2.EC2InstanceDashboardLink",
        ],
        "connection-types": [
            {
                "hook-class-name": "airflow.providers.amazon.aws.hooks.base_aws.AwsGenericHook",
                "connection-type": "aws",
            },
            {
                "hook-class-name": "airflow.providers.amazon.aws.hooks.chime.ChimeWebhookHook",
                "connection-type": "chime",
            },
            {"hook-class-name": "airflow.providers.amazon.aws.hooks.emr.EmrHook", "connection-type": "emr"},
            {
                "hook-class-name": "airflow.providers.amazon.aws.hooks.redshift_sql.RedshiftSQLHook",
                "connection-type": "redshift",
            },
            {
                "hook-class-name": "airflow.providers.amazon.aws.hooks.athena_sql.AthenaSQLHook",
                "connection-type": "athena",
            },
        ],
        "notifications": [
            "airflow.providers.amazon.aws.notifications.chime.ChimeNotifier",
            "airflow.providers.amazon.aws.notifications.sns.SnsNotifier",
            "airflow.providers.amazon.aws.notifications.sqs.SqsNotifier",
        ],
        "secrets-backends": [
            "airflow.providers.amazon.aws.secrets.secrets_manager.SecretsManagerBackend",
            "airflow.providers.amazon.aws.secrets.systems_manager.SystemsManagerParameterStoreBackend",
        ],
        "logging": [
            "airflow.providers.amazon.aws.log.s3_task_handler.S3TaskHandler",
            "airflow.providers.amazon.aws.log.cloudwatch_task_handler.CloudwatchTaskHandler",
        ],
        "config": {
            "aws": {
                "description": "This section contains settings for Amazon Web Services (AWS) integration.",
                "options": {
                    "session_factory": {
                        "description": "Full import path to the class which implements a custom session factory for\n``boto3.session.Session``. For more details please have a look at\n:ref:`howto/connection:aws:session-factory`.\n",
                        "default": None,
                        "example": "my_company.aws.MyCustomSessionFactory",
                        "type": "string",
                        "version_added": "3.1.1",
                    },
                    "cloudwatch_task_handler_json_serializer": {
                        "description": "By default, when logging non-string messages, all non-json objects are logged as `null`.\nExcept `datetime` objects which are ISO formatted. Users can optionally use a `repr` serializer or\nprovide their own JSON serializer for any non-JSON-serializable objects in the logged message.\n\n* `airflow.providers.amazon.aws.log.cloudwatch_task_handler.json_serialize` uses `repr` (be aware\n  there is the potential of logging sensitive data depending on the `repr` method of logged objects)\n* `airflow.providers.amazon.aws.log.cloudwatch_task_handler.json_serialize_legacy` uses `null`.\n\nIf a custom serializer is provided, it must adhere to `Callable[[Any], str | None]`, where `None`\nserializes to `null` (e.g. `def my_serializer(o: Any) -> str | None`). Since this is on the logging\npath and it's possible there's an exception being handled, special care should be taken to fail\ngracefully without raising a new exception inside of your serializer.\n",
                        "type": "string",
                        "version_added": "8.7.2",
                        "example": "airflow.providers.amazon.aws.log.cloudwatch_task_handler.json_serialize",
                        "default": "airflow.providers.amazon.aws.log.cloudwatch_task_handler.json_serialize_legacy",
                    },
                },
            },
            "aws_batch_executor": {
                "description": "This section only applies if you are using the AwsBatchExecutor in\nAirflow's ``[core]`` configuration.\nFor more information on any of these execution parameters, see the link below:\nhttps://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/batch.html#Batch.Client.submit_job\nFor boto3 credential management, see\nhttps://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html\n",
                "options": {
                    "conn_id": {
                        "description": "The Airflow connection (i.e. credentials) used by the Batch executor to make API calls to AWS Batch.\n",
                        "version_added": "8.11",
                        "type": "string",
                        "example": "aws_default",
                        "default": "aws_default",
                    },
                    "region_name": {
                        "description": "The name of the AWS Region where Amazon Batch is configured. Required.\n",
                        "version_added": "8.11",
                        "type": "string",
                        "example": "us-east-1",
                        "default": None,
                    },
                    "max_submit_job_attempts": {
                        "description": "The maximum number of times the Batch Executor should attempt to run a Batch Job.\n",
                        "version_added": "8.11",
                        "type": "integer",
                        "example": "3",
                        "default": "3",
                    },
                    "check_health_on_startup": {
                        "description": "Whether or not to check the Batch Executor health on startup.\n",
                        "version_added": "8.11",
                        "type": "boolean",
                        "example": "True",
                        "default": "True",
                    },
                    "job_name": {
                        "description": "The name of the job submitted to AWS Batch. It can be up to 128 letters long.\nThe first character must be alphanumeric, can contain uppercase and lowercase\nletters, numbers, hyphens (-), and underscores (_).\n",
                        "version_added": "8.11",
                        "type": "string",
                        "example": "airflow-batch-executor-job",
                        "default": None,
                    },
                    "job_queue": {
                        "description": "The job queue where the job is submitted. You can specify either the name\nor the Amazon Resource Name (ARN) of the queue.\n",
                        "version_added": "8.11",
                        "type": "string",
                        "example": "airflow-batch-executor-job-queue",
                        "default": None,
                    },
                    "job_definition": {
                        "description": "The job definition used by the job. You can specify either the name\nor the Amazon Resource Name (ARN) of the job definition with or\nwithout the revision. If the revision is not specified, then the\nlatest active revision is used.\n",
                        "version_added": "8.11",
                        "type": "string",
                        "example": "airflow-batch-executor-job-definition",
                        "default": None,
                    },
                    "submit_job_kwargs": {
                        "description": "Additional parameters to pass to the submit_job method of the\nAWS Batch client.\n",
                        "version_added": "8.11",
                        "type": "string",
                        "example": '{"Tags": [{"Key": "key", "Value": "value"}]}',
                        "default": None,
                    },
                },
            },
            "aws_lambda_executor": {
                "description": "This section only applies if you are using the AwsLambdaExecutor in\nAirflow's ``[core.executor]`` configuration.\nFor more information see:\nhttps://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/lambda/client/invoke.html\nhttps://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html\nhttps://airflow.apache.org/docs/apache-airflow-providers-amazon/stable/executors/lambda-executor.html\n",
                "options": {
                    "conn_id": {
                        "description": "The Airflow connection (i.e. credentials) used by the Lambda executor to make API calls.\n",
                        "version_added": "9.9.0",
                        "type": "string",
                        "example": "aws_default",
                        "default": "aws_default",
                    },
                    "region_name": {
                        "description": "The name of the AWS Region where Amazon Lambda is configured.\n",
                        "version_added": "9.9.0",
                        "type": "string",
                        "example": "us-east-1",
                        "default": None,
                    },
                    "check_health_on_startup": {
                        "description": "Whether or not to check the Lambda Executor health on startup.\n",
                        "version_added": "9.9.0",
                        "type": "boolean",
                        "example": "True",
                        "default": "True",
                    },
                    "max_run_task_attempts": {
                        "description": "The maximum number of times the Lambda Executor should attempt to start an Airflow task.\n",
                        "version_added": "9.9.0",
                        "type": "integer",
                        "example": "3",
                        "default": "3",
                    },
                    "queue_url": {
                        "description": "The URL of the SQS queue to use for the Lambda executor. Required.\nThe Lambda executor will poll this queue for results of the lambda function's Airflow Task.\n",
                        "version_added": "9.9.0",
                        "type": "string",
                        "example": "airflow-lambda-executor-results-queue",
                        "default": None,
                    },
                    "dead_letter_queue_url": {
                        "description": "The URL of the SQS dead letter queue to use for the Lambda function. The Lambda executor\nwill poll this queue for timeout/exception results of the lambda function. Required\n",
                        "version_added": "9.9.0",
                        "type": "string",
                        "example": "airflow-lambda-executor-dlq",
                        "default": None,
                    },
                    "function_name": {
                        "description": "The name of the Lambda function to invoke. Required.\n",
                        "version_added": "9.9.0",
                        "type": "string",
                        "example": "airflow-lambda-executor-function",
                        "default": None,
                    },
                    "qualifier": {
                        "description": "The version or alias of the Lambda function to invoke. If not specified, the latest version is used.\n",
                        "version_added": "9.9.0",
                        "type": "string",
                        "example": "1",
                        "default": None,
                    },
                    "end_wait_timeout": {
                        "description": "The number of seconds to wait for all lambda executor invocations to complete when terminating the\nexecutor/scheduler. Default of 0 means wait indefinitely.\n",
                        "version_added": "9.9.0",
                        "type": "integer",
                        "example": "500",
                        "default": "0",
                    },
                },
            },
            "aws_ecs_executor": {
                "description": "This section only applies if you are using the AwsEcsExecutor in\nAirflow's ``[core]`` configuration.\nFor more information on any of these execution parameters, see the link below:\nhttps://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ecs/client/run_task.html\nFor boto3 credential management, see\nhttps://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html\n",
                "options": {
                    "conn_id": {
                        "description": "The Airflow connection (i.e. credentials) used by the ECS executor to make API calls to AWS ECS.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "aws_default",
                        "default": "aws_default",
                    },
                    "region_name": {
                        "description": "The name of the AWS Region where Amazon ECS is configured. Required.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "us-east-1",
                        "default": None,
                    },
                    "assign_public_ip": {
                        "description": "Whether to assign a public IP address to the containers launched by the ECS executor.\nFor more info see url to Boto3 docs above.\n",
                        "version_added": "8.10",
                        "type": "boolean",
                        "example": "True",
                        "default": "False",
                    },
                    "cluster": {
                        "description": "Name of the Amazon ECS Cluster. Required.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "ecs_executor_cluster",
                        "default": None,
                    },
                    "capacity_provider_strategy": {
                        "description": "The capacity provider strategy to use for the task.\n\nIf a Capacity Provider Strategy is specified, the Launch Type parameter must be omitted. If\nno Capacity Provider Strategy or Launch Type is specified, the Default CapacityProvider Strategy\nfor the cluster is used, if present.\n\nWhen you use cluster auto scaling, you must specify Capacity Provider Strategy and not Launch Type.\n",
                        "version_added": "8.17",
                        "type": "string",
                        "example": "[{'capacityProvider': 'cp1', 'weight': 5}, {'capacityProvider': 'cp2', 'weight': 1}]",
                        "default": None,
                    },
                    "container_name": {
                        "description": "Name of the container that will be used to execute Airflow tasks via the ECS executor.\nThe container should be specified in the ECS Task Definition and will receive an airflow\nCLI command as an additional parameter to its entrypoint. For more info see url to Boto3\ndocs above. Required.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "ecs_executor_container",
                        "default": None,
                    },
                    "launch_type": {
                        "description": "Launch type can either be 'FARGATE' OR 'EC2'. For more info see url to\nBoto3 docs above.\n\nIf a Launch Type is specified, the Capacity Provider Strategy parameter must be omitted. If\nno Capacity Provider Strategy or Launch Type is specified, the Default Capacity Provider Strategy\nfor the cluster is used, if present.\n\nIf the launch type is EC2, the executor will attempt to place tasks on\nempty EC2 instances. If there are no EC2 instances available, no task\nis placed and this function will be called again in the next heart-beat.\n\nIf the launch type is FARGATE, this will run the tasks on new AWS Fargate\ninstances.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "FARGATE",
                        "default": None,
                    },
                    "platform_version": {
                        "description": "The platform version the task uses. A platform version is only specified\nfor tasks hosted on Fargate. If one isn't specified, the LATEST platform\nversion is used.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "1.4.0",
                        "default": "LATEST",
                    },
                    "security_groups": {
                        "description": "The comma-separated IDs of the security groups associated with the task. If you\ndon't specify a security group, the default security group for the VPC is used.\nThere's a limit of 5 security groups. For more info see url to Boto3 docs above.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "sg-XXXX,sg-YYYY",
                        "default": None,
                    },
                    "subnets": {
                        "description": "The comma-separated IDs of the subnets associated with the task or service.\nThere's a limit of 16 subnets. For more info see url to Boto3 docs above.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "subnet-XXXXXXXX,subnet-YYYYYYYY",
                        "default": None,
                    },
                    "task_definition": {
                        "description": "The family and revision (family:revision) or full ARN of the task definition\nto run. If a revision isn't specified, the latest ACTIVE revision is used.\nFor more info see url to Boto3 docs above.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "executor_task_definition:LATEST",
                        "default": None,
                    },
                    "max_run_task_attempts": {
                        "description": "The maximum number of times the ECS Executor should attempt to run a task.\n",
                        "version_added": "8.10",
                        "type": "integer",
                        "example": "3",
                        "default": "3",
                    },
                    "run_task_kwargs": {
                        "description": "A JSON string containing arguments to provide the ECS `run_task` API (see url above).\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": '{"tags": {"key": "schema", "value": "1.0"}}',
                        "default": None,
                    },
                    "check_health_on_startup": {
                        "description": "Whether or not to check the ECS Executor health on startup.\n",
                        "version_added": "8.11",
                        "type": "boolean",
                        "example": "True",
                        "default": "True",
                    },
                },
            },
            "aws_auth_manager": {
                "description": "This section only applies if you are using the AwsAuthManager. In other words, if you set\n``[core] auth_manager = airflow.providers.amazon.aws.auth_manager.aws_auth_manager.AwsAuthManager`` in\nAirflow's configuration.\n",
                "options": {
                    "conn_id": {
                        "description": "The Airflow connection (i.e. credentials) used by the AWS auth manager to make API calls to AWS\nIdentity Center and Amazon Verified Permissions.\n",
                        "version_added": "8.12.0",
                        "type": "string",
                        "example": "aws_default",
                        "default": "aws_default",
                    },
                    "region_name": {
                        "description": "The name of the AWS Region where Amazon Verified Permissions is configured. Required.\n",
                        "version_added": "8.10",
                        "type": "string",
                        "example": "us-east-1",
                        "default": None,
                    },
                    "saml_metadata_url": {
                        "description": "SAML metadata XML file provided by AWS Identity Center.\nThis URL can be found in the AWS Identity Center console. Required.\n",
                        "version_added": "8.12.0",
                        "type": "string",
                        "example": "https://portal.sso.<region>.amazonaws.com/saml/metadata/XXXXXXXXXX",
                        "default": None,
                    },
                    "avp_policy_store_id": {
                        "description": "Amazon Verified Permissions' policy store ID where all the policies defining user permissions\nin Airflow are stored. Required.\n",
                        "version_added": "8.12.0",
                        "type": "string",
                        "example": None,
                        "default": None,
                    },
                },
            },
        },
        "executors": ["airflow.providers.amazon.aws.executors.ecs.ecs_executor.AwsEcsExecutor"],
        "auth-managers": ["airflow.providers.amazon.aws.auth_manager.aws_auth_manager.AwsAuthManager"],
        "cli": ["airflow.providers.amazon.aws.cli.definition.get_aws_cli_commands"],
        "queues": ["airflow.providers.amazon.aws.queues.sqs.SqsMessageQueueProvider"],
    }



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections.abc import AsyncIterator
from typing import TYPE_CHECKING

from airflow.providers.amazon.aws.hooks.ssm import SsmHook
from airflow.providers.amazon.aws.triggers.base import AwsBaseWaiterTrigger
from airflow.providers.amazon.aws.utils.waiter_with_logging import async_wait
from airflow.triggers.base import TriggerEvent

if TYPE_CHECKING:
    from airflow.providers.amazon.aws.hooks.base_aws import AwsGenericHook


class SsmRunCommandTrigger(AwsBaseWaiterTrigger):
    """
    Trigger when a SSM run command is complete.

    :param command_id: The ID of the AWS SSM Run Command.
    :param waiter_delay: The amount of time in seconds to wait between attempts. (default: 120)
    :param waiter_max_attempts: The maximum number of attempts to be made. (default: 75)
    :param fail_on_nonzero_exit: If True (default), the trigger will fail when the command returns
        a non-zero exit code. If False, the trigger will complete successfully regardless of the
        command exit code. (default: True)
    :param aws_conn_id: The Airflow connection used for AWS credentials.
    :param region_name: AWS region_name. If not specified then the default boto3 behaviour is used.
    :param verify: Whether or not to verify SSL certificates. See:
        https://boto3.amazonaws.com/v1/documentation/api/latest/reference/core/session.html
    :param botocore_config: Configuration dictionary (key-values) for botocore client. See:
        https://botocore.amazonaws.com/v1/documentation/api/latest/reference/config.html
    """

    def __init__(
        self,
        *,
        command_id: str,
        waiter_delay: int = 120,
        waiter_max_attempts: int = 75,
        fail_on_nonzero_exit: bool = True,
        aws_conn_id: str | None = None,
        region_name: str | None = None,
        verify: bool | str | None = None,
        botocore_config: dict | None = None,
    ) -> None:
        super().__init__(
            serialized_fields={"command_id": command_id, "fail_on_nonzero_exit": fail_on_nonzero_exit},
            waiter_name="command_executed",
            waiter_args={"CommandId": command_id},
            failure_message="SSM run command failed.",
            status_message="Status of SSM run command is",
            status_queries=["status"],
            return_key="command_id",
            return_value=command_id,
            waiter_delay=waiter_delay,
            waiter_max_attempts=waiter_max_attempts,
            aws_conn_id=aws_conn_id,
            region_name=region_name,
            verify=verify,
            botocore_config=botocore_config,
        )
        self.command_id = command_id
        self.fail_on_nonzero_exit = fail_on_nonzero_exit

    def hook(self) -> AwsGenericHook:
        return SsmHook(
            aws_conn_id=self.aws_conn_id,
            region_name=self.region_name,
            verify=self.verify,
            config=self.botocore_config,
        )

    async def run(self) -> AsyncIterator[TriggerEvent]:
        hook = self.hook()
        async with await hook.get_async_conn() as client:
            response = await client.list_command_invocations(CommandId=self.command_id)
            instance_ids = [invocation["InstanceId"] for invocation in response.get("CommandInvocations", [])]
            waiter = hook.get_waiter(self.waiter_name, deferrable=True, client=client)

            for instance_id in instance_ids:
                self.waiter_args["InstanceId"] = instance_id
                try:
                    await async_wait(
                        waiter,
                        self.waiter_delay,
                        self.attempts,
                        self.waiter_args,
                        self.failure_message,
                        self.status_message,
                        self.status_queries,
                    )
                except Exception:
                    if not self.fail_on_nonzero_exit:
                        # Enhanced mode: check if it's an AWS-level failure
                        invocation = await client.get_command_invocation(
                            CommandId=self.command_id, InstanceId=instance_id
                        )
                        status = invocation.get("Status", "")

                        # AWS-level failures should always raise
                        if SsmHook.is_aws_level_failure(status):
                            raise

                        # Command-level failure - tolerate it in enhanced mode
                        response_code = invocation.get("ResponseCode", "unknown")
                        self.log.info(
                            "Command %s completed with status %s (exit code: %s) for instance %s. "
                            "Continuing due to fail_on_nonzero_exit=False",
                            self.command_id,
                            status,
                            response_code,
                            instance_id,
                        )
                        continue
                    else:
                        # Traditional mode: all failures raise
                        raise

        yield TriggerEvent({"status": "success", self.return_key: self.return_value})



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
from unittest import mock

import pytest
from moto import mock_aws

from airflow.providers.amazon.aws.hooks.sqs import SqsHook
from airflow.providers.amazon.aws.sensors.sqs import SqsSensor
from airflow.providers.common.compat.sdk import AirflowException, TaskDeferred

REGION_NAME = "eu-central-1"
QUEUE_NAME = "test-queue"
QUEUE_URL = f"https://{QUEUE_NAME}"


@pytest.fixture
def mocked_context():
    return mock.MagicMock(name="FakeContext")


@pytest.fixture
def mocked_client():
    with mock.patch(
        "airflow.providers.amazon.aws.hooks.sqs.SqsHook.conn", new_callable=mock.PropertyMock
    ) as m:
        yield m


class TestSqsSensor:
    @pytest.fixture(autouse=True)
    def _setup_test_cases(self):
        self.default_op_kwargs = {
            "task_id": "test_task",
            "aws_conn_id": None,
            "region_name": REGION_NAME,
            "sqs_queue": QUEUE_URL,
        }
        self.sensor = SqsSensor(**self.default_op_kwargs)
        self.sqs_client = SqsHook(aws_conn_id=None, region_name=REGION_NAME).conn

    def test_init(self):
        self.default_op_kwargs.pop("aws_conn_id", None)
        self.default_op_kwargs.pop("region_name", None)

        sensor = SqsSensor(**self.default_op_kwargs)
        assert sensor.hook.aws_conn_id == "aws_default"
        assert sensor.hook._region_name is None
        assert sensor.hook._verify is None
        assert sensor.hook._config is None

        sensor = SqsSensor(
            **self.default_op_kwargs,
            aws_conn_id=None,
            region_name=REGION_NAME,
            verify=True,
            botocore_config={"read_timeout": 42},
        )
        assert sensor.hook.aws_conn_id is None
        assert sensor.hook._region_name == REGION_NAME
        assert sensor.hook._verify is True
        assert sensor.hook._config is not None
        assert sensor.hook._config.read_timeout == 42

    @mock_aws
    def test_poke_success(self, mocked_context):
        self.sqs_client.create_queue(QueueName=QUEUE_NAME)
        self.sqs_client.send_message(QueueUrl=QUEUE_URL, MessageBody="hello")

        assert self.sensor.poke(mocked_context) is True

        mocked_xcom_push = mocked_context["ti"].xcom_push
        assert mocked_xcom_push.call_count == 1
        call_args, call_kwargs = mocked_xcom_push.call_args.args, mocked_xcom_push.call_args.kwargs
        assert not call_args
        assert "key" in call_kwargs
        assert call_kwargs["key"] == "messages"
        assert "value" in call_kwargs
        xcom_value = call_kwargs["value"]
        assert xcom_value[0]["Body"] == "hello"

    @mock_aws
    def test_poke_no_message(self, mocked_context):
        self.sqs_client.create_queue(QueueName=QUEUE_NAME)

        assert self.sensor.poke(mocked_context) is False

        mocked_xcom_push = mocked_context["ti"].xcom_push
        assert mocked_xcom_push.call_count == 0

    def test_poke_delete_raise_airflow_exception(self, mocked_client):
        message = {
            "Messages": [
                {
                    "MessageId": "c585e508-2ea0-44c7-bf3e-d1ba0cb87834",
                    "ReceiptHandle": "mockHandle",
                    "MD5OfBody": "e5a9d8684a8edfed460b8d42fd28842f",
                    "Body": "h21",
                }
            ],
            "ResponseMetadata": {
                "RequestId": "56cbf4aa-f4ef-5518-9574-a04e0a5f1411",
                "HTTPStatusCode": 200,
                "HTTPHeaders": {
                    "x-amzn-requestid": "56cbf4aa-f4ef-5518-9574-a04e0a5f1411",
                    "date": "Mon, 18 Feb 2019 18:41:52 GMT",
                    "content-type": "text/xml",
                    "mock_sqs_hook-length": "830",
                },
                "RetryAttempts": 0,
            },
        }
        mocked_client.return_value.receive_message.return_value = message
        mocked_client.return_value.delete_message_batch.return_value = {
            "Failed": [{"Id": "22f67273-4dbc-4c19-83b5-aee71bfeb832"}]
        }

        with pytest.raises(AirflowException, match="Delete SQS Messages failed"):
            self.sensor.poke({})

    def test_poke_receive_raise_exception(self, mocked_client):
        mocked_client.return_value.receive_message.side_effect = Exception("test exception")
        with pytest.raises(Exception, match="test exception"):
            self.sensor.poke({})

    def test_poke_visibility_timeout(self, mocked_client, mocked_context):
        # Check without visibility_timeout parameter
        self.sensor.poke(mocked_context)
        calls_receive_message = [
            mock.call().receive_message(QueueUrl=QUEUE_URL, MaxNumberOfMessages=5, WaitTimeSeconds=1)
        ]
        mocked_client.assert_has_calls(calls_receive_message)

        # Check with visibility_timeout parameter
        SqsSensor(**self.default_op_kwargs, visibility_timeout=42).poke(mocked_context)
        calls_receive_message = [
            mock.call().receive_message(
                QueueUrl=QUEUE_URL, MaxNumberOfMessages=5, WaitTimeSeconds=1, VisibilityTimeout=42
            )
        ]
        mocked_client.assert_has_calls(calls_receive_message)

    @mock_aws
    def test_poke_message_invalid_filtering(self):
        self.sqs_client.create_queue(QueueName=QUEUE_NAME)
        self.sqs_client.send_message(QueueUrl=QUEUE_URL, MessageBody="hello")

        sensor = SqsSensor(**self.default_op_kwargs, message_filtering="invalid_option")
        with pytest.raises(NotImplementedError, match="Override this method to define custom filters"):
            sensor.poke({})

    def test_poke_message_filtering_literal_values(self, mocked_client, mocked_context):
        matching = [{"id": 11, "body": "a matching message"}]
        non_matching = [{"id": 12, "body": "a non-matching message"}]

        def mock_receive_message(**kwargs):
            messages = []
            for message in [*matching, *non_matching]:
                messages.append(
                    {
                        "MessageId": message["id"],
                        "ReceiptHandle": 100 + message["id"],
                        "Body": message["body"],
                    }
                )
            return {"Messages": messages}

        mocked_client.return_value.receive_message.side_effect = mock_receive_message

        def mock_delete_message_batch(**kwargs):
            return {"Successful"}

        mocked_client.return_value.delete_message_batch.side_effect = mock_delete_message_batch

        # Test that messages are filtered
        sensor = SqsSensor(
            **self.default_op_kwargs,
            message_filtering="literal",
            message_filtering_match_values=["a matching message"],
        )
        assert sensor.poke(mocked_context) is True

        # Test that only filtered messages are deleted
        delete_entries = [{"Id": x["id"], "ReceiptHandle": 100 + x["id"]} for x in matching]
        calls_delete_message_batch = [
            mock.call().delete_message_batch(QueueUrl=QUEUE_URL, Entries=delete_entries)
        ]
        mocked_client.assert_has_calls(calls_delete_message_batch)

    def test_poke_message_filtering_jsonpath(self, mocked_client, mocked_context):
        matching = [
            {"id": 11, "key": {"matches": [1, 2]}},
            {"id": 12, "key": {"matches": [3, 4, 5]}},
            {"id": 13, "key": {"matches": [10]}},
        ]
        non_matching = [
            {"id": 14, "key": {"nope": [5, 6]}},
            {"id": 15, "key": {"nope": [7, 8]}},
        ]

        def mock_receive_message(**kwargs):
            messages = []
            for message in [*matching, *non_matching]:
                messages.append(
                    {
                        "MessageId": message["id"],
                        "ReceiptHandle": 100 + message["id"],
                        "Body": json.dumps(message),
                    }
                )
            return {"Messages": messages}

        mocked_client.return_value.receive_message.side_effect = mock_receive_message

        def mock_delete_message_batch(**kwargs):
            return {"Successful"}

        mocked_client.return_value.delete_message_batch.side_effect = mock_delete_message_batch

        # Test that messages are filtered
        sensor = SqsSensor(
            **self.default_op_kwargs, message_filtering="jsonpath", message_filtering_config="key.matches[*]"
        )
        assert sensor.poke(mocked_context)

        # Test that only filtered messages are deleted
        delete_entries = [{"Id": x["id"], "ReceiptHandle": 100 + x["id"]} for x in matching]
        calls_delete_message_batch = [
            mock.call().delete_message_batch(QueueUrl=QUEUE_URL, Entries=delete_entries)
        ]
        mocked_client.assert_has_calls(calls_delete_message_batch)

    def test_poke_message_filtering_jsonpath_values(self, mocked_client, mocked_context):
        matching = [
            {"id": 11, "key": {"matches": [1, 2]}},
            {"id": 12, "key": {"matches": [1, 4, 5]}},
            {"id": 13, "key": {"matches": [4, 5]}},
        ]
        non_matching = [
            {"id": 21, "key": {"matches": [10]}},
            {"id": 22, "key": {"nope": [5, 6]}},
            {"id": 23, "key": {"nope": [7, 8]}},
        ]

        def mock_receive_message(**kwargs):
            messages = []
            for message in [*matching, *non_matching]:
                messages.append(
                    {
                        "MessageId": message["id"],
                        "ReceiptHandle": 100 + message["id"],
                        "Body": json.dumps(message),
                    }
                )
            return {"Messages": messages}

        mocked_client.return_value.receive_message.side_effect = mock_receive_message

        def mock_delete_message_batch(**kwargs):
            return {"Successful"}

        mocked_client.return_value.delete_message_batch.side_effect = mock_delete_message_batch

        # Test that messages are filtered
        sensor = SqsSensor(
            **self.default_op_kwargs,
            message_filtering="jsonpath",
            message_filtering_config="key.matches[*]",
            message_filtering_match_values=[1, 4],
        )
        assert sensor.poke(mocked_context)

        # Test that only filtered messages are deleted
        delete_entries = [{"Id": x["id"], "ReceiptHandle": 100 + x["id"]} for x in matching]
        calls_delete_message_batch = [
            mock.call().delete_message_batch(QueueUrl="https://test-queue", Entries=delete_entries)
        ]
        mocked_client.assert_has_calls(calls_delete_message_batch)

    def test_poke_message_filtering_jsonpath_ext(self, mocked_client, mocked_context):
        matching = [
            {"id": 11, "key": "a", "value": "b"},
        ]
        non_matching = [
            {"id": 14, "key": "a"},
            {"id": 14, "value": "b"},
        ]

        def mock_receive_message(**kwargs):
            messages = []
            for message in [*matching, *non_matching]:
                messages.append(
                    {
                        "MessageId": message["id"],
                        "ReceiptHandle": 100 + message["id"],
                        "Body": json.dumps(message),
                    }
                )
            return {"Messages": messages}

        mocked_client.return_value.receive_message.side_effect = mock_receive_message

        def mock_delete_message_batch(**kwargs):
            return {"Successful"}

        mocked_client.return_value.delete_message_batch.side_effect = mock_delete_message_batch

        # Test that messages are filtered
        sensor = SqsSensor(
            **self.default_op_kwargs,
            message_filtering="jsonpath-ext",
            message_filtering_config="$.key + $.value",
        )
        assert sensor.poke(mocked_context)

        # Test that only filtered messages are deleted
        delete_entries = [{"Id": x["id"], "ReceiptHandle": 100 + x["id"]} for x in matching]
        calls_delete_message_batch = [
            mock.call().delete_message_batch(QueueUrl=QUEUE_URL, Entries=delete_entries)
        ]
        mocked_client.assert_has_calls(calls_delete_message_batch)

    def test_poke_message_filtering_jsonpath_ext_values(self, mocked_client, mocked_context):
        matching = [
            {"id": 11, "key": "a1", "value": "b1"},
        ]
        non_matching = [
            {"id": 22, "key": "a2", "value": "b1"},
            {"id": 33, "key": "a1", "value": "b2"},
        ]

        def mock_receive_message(**kwargs):
            messages = []
            for message in [*matching, *non_matching]:
                messages.append(
                    {
                        "MessageId": message["id"],
                        "ReceiptHandle": 100 + message["id"],
                        "Body": json.dumps(message),
                    }
                )
            return {"Messages": messages}

        mocked_client.return_value.receive_message.side_effect = mock_receive_message

        def mock_delete_message_batch(**kwargs):
            return {"Successful"}

        mocked_client.return_value.delete_message_batch.side_effect = mock_delete_message_batch

        # Test that messages are filtered
        sensor = SqsSensor(
            **self.default_op_kwargs,
            message_filtering="jsonpath-ext",
            message_filtering_config="$.key + ' ' + $.value",
            message_filtering_match_values=["a1 b1"],
        )
        assert sensor.poke(mocked_context)

        # Test that only filtered messages are deleted
        delete_entries = [{"Id": x["id"], "ReceiptHandle": 100 + x["id"]} for x in matching]
        calls_delete_message_batch = [
            mock.call().delete_message_batch(QueueUrl="https://test-queue", Entries=delete_entries)
        ]
        mocked_client.assert_has_calls(calls_delete_message_batch)

    def test_poke_do_not_delete_message_on_received(self, mocked_client, mocked_context):
        sensor = SqsSensor(**self.default_op_kwargs, delete_message_on_reception=False)
        sensor.poke(mocked_context)
        assert mocked_client.delete_message_batch.called is False

    @mock_aws
    def test_poke_batch_messages(self, mocked_context):
        messages = ["hello", "brave", "world"]
        self.sqs_client.create_queue(QueueName=QUEUE_NAME)
        # Do publish 3 messages
        for message in messages:
            self.sqs_client.send_message(QueueUrl=QUEUE_URL, MessageBody=message)

        # Init batch sensor to get 1 message for each SQS poll and perform 3 polls
        sensor = SqsSensor(**self.default_op_kwargs, max_messages=1, num_batches=3)
        assert sensor.poke(mocked_context)

        # expect all messages are retrieved
        mocked_xcom_push = mocked_context["ti"].xcom_push
        assert mocked_xcom_push.call_count == 1
        call_args, call_kwargs = mocked_xcom_push.call_args.args, mocked_xcom_push.call_args.kwargs
        assert not call_args
        assert "key" in call_kwargs
        assert call_kwargs["key"] == "messages"
        assert "value" in call_kwargs
        xcom_value = call_kwargs["value"]
        assert len(xcom_value) == 3
        received_messages = list(map(lambda x: x["Body"], xcom_value))
        for message in messages:
            assert message in received_messages

    def test_sqs_deferrable(self):
        sensor = SqsSensor(**self.default_op_kwargs, deferrable=True)
        with pytest.raises(TaskDeferred):
            sensor.execute({})

    def test_fail_execute_complete(self):
        sensor = SqsSensor(**self.default_op_kwargs, deferrable=True)
        event = {"status": "failed"}
        message = f"Trigger error: event is {event}"
        with pytest.raises(AirflowException, match=message):
            sensor.execute_complete(context={}, event=event)

    @mock.patch("airflow.providers.amazon.aws.sensors.sqs.SqsSensor.poll_sqs")
    @mock.patch("airflow.providers.amazon.aws.sensors.sqs.process_response")
    @mock.patch("airflow.providers.amazon.aws.hooks.sqs.SqsHook.conn")
    def test_fail_poke(self, mocked_client, process_response, poll_sqs):
        response = "error message"
        messages = [{"MessageId": "1", "ReceiptHandle": "test"}]
        poll_sqs.return_value = response
        process_response.return_value = messages
        mocked_client.delete_message_batch.return_value = response
        error_message = f"Delete SQS Messages failed {response} for messages"

        sensor = SqsSensor(**self.default_op_kwargs)
        with pytest.raises(AirflowException, match=error_message):
            sensor.poke(context={})



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import logging
import time
from unittest import mock

import botocore.exceptions
import pytest

from airflow.providers.amazon.aws.hooks.batch_client import BatchClientHook
from airflow.providers.amazon.aws.utils.task_log_fetcher import AwsTaskLogFetcher
from airflow.providers.common.compat.sdk import AirflowException

# Use dummy AWS credentials
AWS_REGION = "eu-west-1"
AWS_ACCESS_KEY_ID = "airflow_dummy_key"
AWS_SECRET_ACCESS_KEY = "airflow_dummy_secret"

JOB_ID = "8ba9d676-4108-4474-9dca-8bbac1da9b19"
LOG_STREAM_NAME = "test/stream/d56a66bb98a14c4593defa1548686edf"


class TestBatchClient:
    MAX_RETRIES = 2
    STATUS_RETRIES = 3

    @mock.patch.dict("os.environ", AWS_DEFAULT_REGION=AWS_REGION)
    @mock.patch.dict("os.environ", AWS_ACCESS_KEY_ID=AWS_ACCESS_KEY_ID)
    @mock.patch.dict("os.environ", AWS_SECRET_ACCESS_KEY=AWS_SECRET_ACCESS_KEY)
    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.AwsBaseHook.get_client_type")
    def setup_method(self, method, get_client_type_mock):
        self.get_client_type_mock = get_client_type_mock
        self.batch_client = BatchClientHook(
            max_retries=self.MAX_RETRIES,
            status_retries=self.STATUS_RETRIES,
            aws_conn_id="airflow_test",
            region_name=AWS_REGION,
        )
        # We're mocking all actual AWS calls and don't need a connection. This
        # avoids an Airflow warning about connection cannot be found.
        self.batch_client.get_connection = lambda _: None
        self.client_mock = get_client_type_mock.return_value
        assert self.batch_client.client == self.client_mock  # setup client property

        # don't pause in these unit tests
        self.mock_delay = mock.Mock(return_value=None)
        self.batch_client.delay = self.mock_delay
        self.mock_exponential_delay = mock.Mock(return_value=0)
        self.batch_client.exponential_delay = self.mock_exponential_delay

    def test_init(self):
        assert self.batch_client.max_retries == self.MAX_RETRIES
        assert self.batch_client.status_retries == self.STATUS_RETRIES
        assert self.batch_client.region_name == AWS_REGION
        assert self.batch_client.aws_conn_id == "airflow_test"
        assert self.batch_client.client == self.client_mock

        self.get_client_type_mock.assert_called_once_with(region_name=AWS_REGION)

    def test_wait_for_job_with_success(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": "SUCCEEDED"}]}

        with mock.patch.object(
            self.batch_client,
            "poll_for_job_running",
            wraps=self.batch_client.poll_for_job_running,
        ) as job_running:
            self.batch_client.wait_for_job(JOB_ID)
            job_running.assert_called_once_with(JOB_ID, None)

        with mock.patch.object(
            self.batch_client,
            "poll_for_job_complete",
            wraps=self.batch_client.poll_for_job_complete,
        ) as job_complete:
            self.batch_client.wait_for_job(JOB_ID)
            job_complete.assert_called_once_with(JOB_ID, None)

        assert self.client_mock.describe_jobs.call_count == 4

    def test_wait_for_job_with_failure(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": "FAILED"}]}

        with mock.patch.object(
            self.batch_client,
            "poll_for_job_running",
            wraps=self.batch_client.poll_for_job_running,
        ) as job_running:
            self.batch_client.wait_for_job(JOB_ID)
            job_running.assert_called_once_with(JOB_ID, None)

        with mock.patch.object(
            self.batch_client,
            "poll_for_job_complete",
            wraps=self.batch_client.poll_for_job_complete,
        ) as job_complete:
            self.batch_client.wait_for_job(JOB_ID)
            job_complete.assert_called_once_with(JOB_ID, None)

        assert self.client_mock.describe_jobs.call_count == 4

    def test_wait_for_job_with_logs(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": "SUCCEEDED"}]}

        batch_log_fetcher = mock.Mock(spec=AwsTaskLogFetcher)
        mock_get_batch_log_fetcher = mock.Mock(return_value=batch_log_fetcher)

        thread_start = mock.Mock(side_effect=lambda: time.sleep(2))
        thread_stop = mock.Mock(side_effect=lambda: time.sleep(2))
        thread_join = mock.Mock(side_effect=lambda: time.sleep(2))

        with (
            mock.patch.object(batch_log_fetcher, "start", thread_start) as mock_fetcher_start,
            mock.patch.object(batch_log_fetcher, "stop", thread_stop) as mock_fetcher_stop,
            mock.patch.object(batch_log_fetcher, "join", thread_join) as mock_fetcher_join,
        ):
            self.batch_client.wait_for_job(JOB_ID, get_batch_log_fetcher=mock_get_batch_log_fetcher)
            mock_get_batch_log_fetcher.assert_called_with(JOB_ID)
            mock_fetcher_start.assert_called_once()
            mock_fetcher_stop.assert_called_once()
            mock_fetcher_join.assert_called_once()

    def test_poll_job_running_for_status_running(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": "RUNNING"}]}
        self.batch_client.poll_for_job_running(JOB_ID)
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])

    def test_poll_job_complete_for_status_success(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": "SUCCEEDED"}]}
        self.batch_client.poll_for_job_complete(JOB_ID)
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])

    def test_poll_job_complete_raises_for_max_retries(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": "RUNNING"}]}
        with pytest.raises(AirflowException) as ctx:
            self.batch_client.poll_for_job_complete(JOB_ID)
        msg = f"AWS Batch job ({JOB_ID}) status checks exceed max_retries"
        assert msg in str(ctx.value)
        self.client_mock.describe_jobs.assert_called_with(jobs=[JOB_ID])
        assert self.client_mock.describe_jobs.call_count == self.MAX_RETRIES + 1

    def test_poll_job_status_hit_api_throttle(self, caplog):
        self.client_mock.describe_jobs.side_effect = botocore.exceptions.ClientError(
            error_response={"Error": {"Code": "TooManyRequestsException"}},
            operation_name="get job description",
        )
        with pytest.raises(AirflowException) as ctx:
            with caplog.at_level(level=logging.getLevelName("WARNING")):
                self.batch_client.poll_for_job_complete(JOB_ID)
        log_record = caplog.records[0]
        assert "Ignored TooManyRequestsException error" in log_record.message

        msg = f"AWS Batch job ({JOB_ID}) description error"
        assert msg in str(ctx.value)
        # It should retry when this client error occurs
        self.client_mock.describe_jobs.assert_called_with(jobs=[JOB_ID])
        assert self.client_mock.describe_jobs.call_count == self.STATUS_RETRIES

    def test_poll_job_status_with_client_error(self):
        self.client_mock.describe_jobs.side_effect = botocore.exceptions.ClientError(
            error_response={"Error": {"Code": "InvalidClientTokenId"}},
            operation_name="get job description",
        )
        with pytest.raises(botocore.exceptions.ClientError) as ctx:
            self.batch_client.poll_for_job_complete(JOB_ID)

        assert ctx.value.response["Error"]["Code"] == "InvalidClientTokenId"
        # It will not retry when this client error occurs
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])

    def test_check_job_success(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": "SUCCEEDED"}]}
        status = self.batch_client.check_job_success(JOB_ID)
        assert status
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])

    def test_check_job_success_raises_failed(self):
        self.client_mock.describe_jobs.return_value = {
            "jobs": [
                {
                    "jobId": JOB_ID,
                    "status": "FAILED",
                    "statusReason": "This is an error reason",
                    "attempts": [{"exitCode": 1}],
                }
            ]
        }
        with pytest.raises(AirflowException) as ctx:
            self.batch_client.check_job_success(JOB_ID)
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])
        msg = f"AWS Batch job ({JOB_ID}) failed"
        assert msg in str(ctx.value)

    def test_check_job_success_raises_failed_for_multiple_attempts(self):
        self.client_mock.describe_jobs.return_value = {
            "jobs": [
                {
                    "jobId": JOB_ID,
                    "status": "FAILED",
                    "statusReason": "This is an error reason",
                    "attempts": [{"exitCode": 1}, {"exitCode": 10}],
                }
            ]
        }
        with pytest.raises(AirflowException) as ctx:
            self.batch_client.check_job_success(JOB_ID)
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])
        msg = f"AWS Batch job ({JOB_ID}) failed"
        assert msg in str(ctx.value)

    def test_check_job_success_raises_incomplete(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": "RUNNABLE"}]}
        with pytest.raises(AirflowException) as ctx:
            self.batch_client.check_job_success(JOB_ID)
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])
        msg = f"AWS Batch job ({JOB_ID}) is not complete"
        assert msg in str(ctx.value)

    def test_check_job_success_raises_unknown_status(self):
        status = "STRANGE"
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID, "status": status}]}
        with pytest.raises(AirflowException) as ctx:
            self.batch_client.check_job_success(JOB_ID)
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])
        msg = f"AWS Batch job ({JOB_ID}) has unknown status"
        assert msg in str(ctx.value)
        assert status in str(ctx.value)

    def test_check_job_success_raises_without_jobs(self, caplog):
        self.client_mock.describe_jobs.return_value = {"jobs": []}
        with caplog.at_level(level=logging.WARNING):
            with pytest.raises(AirflowException):
                self.batch_client.check_job_success(JOB_ID)
            self.client_mock.describe_jobs.assert_has_calls([mock.call(jobs=[JOB_ID])] * 3)
            msg = f"AWS Batch job ({JOB_ID}) description error"
            assert msg in caplog.messages[0]

    def test_terminate_job(self):
        self.client_mock.terminate_job.return_value = {}
        reason = "Task killed by the user"
        response = self.batch_client.terminate_job(JOB_ID, reason)
        self.client_mock.terminate_job.assert_called_once_with(jobId=JOB_ID, reason=reason)
        assert response == {}

    def test_job_awslogs_default(self):
        self.client_mock.describe_jobs.return_value = {
            "jobs": [
                {
                    "jobId": JOB_ID,
                    "container": {"logStreamName": LOG_STREAM_NAME},
                }
            ]
        }
        self.client_mock.meta.client.meta.region_name = AWS_REGION

        awslogs = self.batch_client.get_job_awslogs_info(JOB_ID)
        assert awslogs["awslogs_stream_name"] == LOG_STREAM_NAME
        assert awslogs["awslogs_group"] == "/aws/batch/job"
        assert awslogs["awslogs_region"] == AWS_REGION

    def test_job_awslogs_user_defined(self):
        self.client_mock.describe_jobs.return_value = {
            "jobs": [
                {
                    "jobId": JOB_ID,
                    "container": {
                        "logStreamName": LOG_STREAM_NAME,
                        "logConfiguration": {
                            "logDriver": "awslogs",
                            "options": {
                                "awslogs-group": "/test/batch/job",
                                "awslogs-region": "ap-southeast-2",
                            },
                        },
                    },
                }
            ]
        }
        awslogs = self.batch_client.get_job_awslogs_info(JOB_ID)
        assert awslogs["awslogs_stream_name"] == LOG_STREAM_NAME
        assert awslogs["awslogs_group"] == "/test/batch/job"
        assert awslogs["awslogs_region"] == "ap-southeast-2"

    def test_job_no_awslogs_stream(self, caplog):
        self.client_mock.describe_jobs.return_value = {
            "jobs": [
                {
                    "jobId": JOB_ID,
                    "container": {"logConfiguration": {}},
                }
            ]
        }

        with caplog.at_level(level=logging.WARNING):
            assert self.batch_client.get_job_awslogs_info(JOB_ID) is None
            assert len(caplog.records) == 1
            assert "doesn't have any AWS CloudWatch Stream" in caplog.messages[0]

    def test_job_not_recognized_job(self):
        self.client_mock.describe_jobs.return_value = {"jobs": [{"jobId": JOB_ID}]}
        with pytest.raises(AirflowException) as ctx:
            self.batch_client.get_job_awslogs_info(JOB_ID)
        # It should not retry when this client error occurs
        self.client_mock.describe_jobs.assert_called_once_with(jobs=[JOB_ID])
        msg = "is not a supported job type"
        assert msg in str(ctx.value)

    def test_job_splunk_logs(self, caplog):
        self.client_mock.describe_jobs.return_value = {
            "jobs": [
                {
                    "jobId": JOB_ID,
                    "logStreamName": LOG_STREAM_NAME,
                    "container": {
                        "logConfiguration": {
                            "logDriver": "splunk",
                        }
                    },
                }
            ]
        }
        with caplog.at_level(level=logging.WARNING):
            assert self.batch_client.get_job_awslogs_info(JOB_ID) is None
            assert len(caplog.records) == 1
            assert "uses non-aws log drivers. AWS CloudWatch logging disabled." in caplog.messages[0]

    def test_job_awslogs_multinode_job(self):
        self.client_mock.describe_jobs.return_value = {
            "jobs": [
                {
                    "jobId": JOB_ID,
                    "attempts": [
                        {"container": {"exitCode": 0, "logStreamName": "test/stream/attempt0"}},
                        {"container": {"exitCode": 0, "logStreamName": "test/stream/attempt1"}},
                    ],
                    "nodeProperties": {
                        "mainNode": 0,
                        "nodeRangeProperties": [
                            {
                                "targetNodes": "0:",
                                "container": {
                                    "logConfiguration": {
                                        "logDriver": "awslogs",
                                        "options": {
                                            "awslogs-group": "/test/batch/job-a",
                                            "awslogs-region": AWS_REGION,
                                        },
                                    }
                                },
                            },
                            {
                                "targetNodes": "1:",
                                "container": {
                                    "logConfiguration": {
                                        "logDriver": "awslogs",
                                        "options": {
                                            "awslogs-group": "/test/batch/job-b",
                                            "awslogs-region": AWS_REGION,
                                        },
                                    }
                                },
                            },
                        ],
                    },
                }
            ]
        }
        awslogs = self.batch_client.get_job_all_awslogs_info(JOB_ID)
        assert len(awslogs) == 4
        assert all([log["awslogs_region"] == AWS_REGION for log in awslogs])

        combinations = {
            ("test/stream/attempt0", "/test/batch/job-a"): False,
            ("test/stream/attempt0", "/test/batch/job-b"): False,
            ("test/stream/attempt1", "/test/batch/job-a"): False,
            ("test/stream/attempt1", "/test/batch/job-b"): False,
        }
        for log_info in awslogs:
            # mark combinations that we see
            combinations[(log_info["awslogs_stream_name"], log_info["awslogs_group"])] = True

        assert len(combinations) == 4
        # all combinations listed above should have been seen
        assert all(combinations.values())


class TestBatchClientDelays:
    @mock.patch.dict("os.environ", AWS_DEFAULT_REGION=AWS_REGION)
    @mock.patch.dict("os.environ", AWS_ACCESS_KEY_ID=AWS_ACCESS_KEY_ID)
    @mock.patch.dict("os.environ", AWS_SECRET_ACCESS_KEY=AWS_SECRET_ACCESS_KEY)
    def setup_method(self, method):
        self.batch_client = BatchClientHook(aws_conn_id="airflow_test", region_name=AWS_REGION)
        # We're mocking all actual AWS calls and don't need a connection. This
        # avoids an Airflow warning about connection cannot be found.
        self.batch_client.get_connection = lambda _: None

    def test_init(self):
        assert self.batch_client.max_retries == self.batch_client.MAX_RETRIES
        assert self.batch_client.status_retries == self.batch_client.STATUS_RETRIES
        assert self.batch_client.region_name == AWS_REGION
        assert self.batch_client.aws_conn_id == "airflow_test"

    def test_add_jitter(self):
        minima = 0
        width = 5
        result = self.batch_client.add_jitter(0, width=width, minima=minima)
        assert result >= minima
        assert result <= width

    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.random.uniform")
    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.time.sleep")
    def test_delay_defaults(self, mock_sleep, mock_uniform):
        assert BatchClientHook.DEFAULT_DELAY_MIN == 1
        assert BatchClientHook.DEFAULT_DELAY_MAX == 10
        mock_uniform.return_value = 0
        self.batch_client.delay()
        mock_uniform.assert_called_once_with(
            BatchClientHook.DEFAULT_DELAY_MIN, BatchClientHook.DEFAULT_DELAY_MAX
        )
        mock_sleep.assert_called_once_with(0)

    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.random.uniform")
    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.time.sleep")
    def test_delay_with_zero(self, mock_sleep, mock_uniform):
        self.batch_client.delay(0)
        mock_uniform.assert_called_once_with(0, 1)  # in add_jitter
        mock_sleep.assert_called_once_with(mock_uniform.return_value)

    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.random.uniform")
    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.time.sleep")
    def test_delay_with_int(self, mock_sleep, mock_uniform):
        self.batch_client.delay(5)
        mock_uniform.assert_called_once_with(4, 6)  # in add_jitter
        mock_sleep.assert_called_once_with(mock_uniform.return_value)

    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.random.uniform")
    @mock.patch("airflow.providers.amazon.aws.hooks.batch_client.time.sleep")
    def test_delay_with_float(self, mock_sleep, mock_uniform):
        self.batch_client.delay(5.0)
        mock_uniform.assert_called_once_with(4.0, 6.0)  # in add_jitter
        mock_sleep.assert_called_once_with(mock_uniform.return_value)

    @pytest.mark.parametrize(
        ("tries", "lower", "upper"),
        [
            (0, 0, 1),
            (1, 0, 2),
            (2, 0, 3),
            (3, 1, 5),
            (4, 2, 7),
            (5, 3, 11),
            (6, 4, 14),
            (7, 6, 19),
            (8, 8, 25),
            (9, 10, 31),
            (45, 200, 600),  # > 40 tries invokes maximum delay allowed
        ],
    )
    def test_exponential_delay(self, tries, lower, upper):
        result = self.batch_client.exponential_delay(tries)
        assert result >= lower
        assert result <= upper



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

import json
from datetime import datetime

from airflow.providers.amazon.aws.hooks.step_function import StepFunctionHook
from airflow.providers.amazon.aws.operators.step_function import (
    StepFunctionGetExecutionOutputOperator,
    StepFunctionStartExecutionOperator,
)
from airflow.providers.amazon.aws.sensors.step_function import StepFunctionExecutionSensor

from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS

if AIRFLOW_V_3_0_PLUS:
    from airflow.sdk import DAG, chain, task
else:
    # Airflow 2 path
    from airflow.decorators import task  # type: ignore[attr-defined,no-redef]
    from airflow.models.baseoperator import chain  # type: ignore[attr-defined,no-redef]
    from airflow.models.dag import DAG  # type: ignore[attr-defined,no-redef,assignment]


from system.amazon.aws.utils import ENV_ID_KEY, SystemTestContextBuilder

DAG_ID = "example_step_functions"

# Externally fetched variables:
ROLE_ARN_KEY = "ROLE_ARN"

sys_test_context_task = SystemTestContextBuilder().add_variable(ROLE_ARN_KEY).build()

STATE_MACHINE_DEFINITION = {
    "StartAt": "Wait",
    "States": {"Wait": {"Type": "Wait", "Seconds": 7, "Next": "Success"}, "Success": {"Type": "Succeed"}},
}


@task
def create_state_machine(env_id, role_arn):
    # Create a Step Functions State Machine and return the ARN for use by
    # downstream tasks.
    return (
        StepFunctionHook()
        .get_conn()
        .create_state_machine(
            name=f"{DAG_ID}_{env_id}",
            definition=json.dumps(STATE_MACHINE_DEFINITION),
            roleArn=role_arn,
        )["stateMachineArn"]
    )


@task
def delete_state_machine(state_machine_arn):
    StepFunctionHook().get_conn().delete_state_machine(stateMachineArn=state_machine_arn)


with DAG(
    dag_id=DAG_ID,
    schedule="@once",
    start_date=datetime(2021, 1, 1),
    catchup=False,
) as dag:
    # This context contains the ENV_ID and any env variables requested when the
    # task was built above. Access the info as you would any other TaskFlow task.
    test_context = sys_test_context_task()
    env_id = test_context[ENV_ID_KEY]
    role_arn = test_context[ROLE_ARN_KEY]

    state_machine_arn = create_state_machine(env_id, role_arn)

    # [START howto_operator_step_function_start_execution]
    start_execution = StepFunctionStartExecutionOperator(
        task_id="start_execution", state_machine_arn=state_machine_arn
    )
    # [END howto_operator_step_function_start_execution]

    execution_arn = start_execution.output

    # [START howto_sensor_step_function_execution]
    wait_for_execution = StepFunctionExecutionSensor(
        task_id="wait_for_execution", execution_arn=execution_arn
    )
    # [END howto_sensor_step_function_execution]
    wait_for_execution.poke_interval = 1

    # [START howto_operator_step_function_get_execution_output]
    get_execution_output = StepFunctionGetExecutionOutputOperator(
        task_id="get_execution_output", execution_arn=execution_arn
    )
    # [END howto_operator_step_function_get_execution_output]

    chain(
        # TEST SETUP
        test_context,
        state_machine_arn,
        # TEST BODY
        start_execution,
        wait_for_execution,
        get_execution_output,
        # TEST TEARDOWN
        delete_state_machine(state_machine_arn),
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from __future__ import annotations

from datetime import datetime

from airflow.providers.amazon.aws.operators.emr import (
    EmrStartNotebookExecutionOperator,
    EmrStopNotebookExecutionOperator,
)
from airflow.providers.amazon.aws.sensors.emr import EmrNotebookExecutionSensor
from airflow.providers.common.compat.sdk import DAG, chain

from system.amazon.aws.utils import ENV_ID_KEY, SystemTestContextBuilder

DAG_ID = "example_emr_notebook"
# Externally fetched variables:
EDITOR_ID_KEY = "EDITOR_ID"
CLUSTER_ID_KEY = "CLUSTER_ID"

sys_test_context_task = (
    SystemTestContextBuilder().add_variable(EDITOR_ID_KEY).add_variable(CLUSTER_ID_KEY).build()
)

with DAG(
    dag_id=DAG_ID,
    start_date=datetime(2021, 1, 1),
    schedule="@once",
    catchup=False,
) as dag:
    test_context = sys_test_context_task()
    env_id = test_context[ENV_ID_KEY]
    editor_id = test_context[EDITOR_ID_KEY]
    cluster_id = test_context[CLUSTER_ID_KEY]

    # [START howto_operator_emr_start_notebook_execution]
    start_execution = EmrStartNotebookExecutionOperator(
        task_id="start_execution",
        editor_id=editor_id,
        cluster_id=cluster_id,
        relative_path="EMR-System-Test.ipynb",
        service_role="EMR_Notebooks_DefaultRole",
    )
    # [END howto_operator_emr_start_notebook_execution]

    notebook_execution_id_1 = start_execution.output

    # [START howto_sensor_emr_notebook_execution]
    wait_for_execution_start = EmrNotebookExecutionSensor(
        task_id="wait_for_execution_start",
        notebook_execution_id=notebook_execution_id_1,
        target_states={"RUNNING"},
        poke_interval=5,
    )
    # [END howto_sensor_emr_notebook_execution]

    # [START howto_operator_emr_stop_notebook_execution]
    stop_execution = EmrStopNotebookExecutionOperator(
        task_id="stop_execution",
        notebook_execution_id=notebook_execution_id_1,
    )
    # [END howto_operator_emr_stop_notebook_execution]

    wait_for_execution_stop = EmrNotebookExecutionSensor(
        task_id="wait_for_execution_stop",
        notebook_execution_id=notebook_execution_id_1,
        target_states={"STOPPED"},
        poke_interval=5,
    )
    finish_execution = EmrStartNotebookExecutionOperator(
        task_id="finish_execution",
        editor_id=editor_id,
        cluster_id=cluster_id,
        relative_path="EMR-System-Test.ipynb",
        service_role="EMR_Notebooks_DefaultRole",
    )
    notebook_execution_id_2 = finish_execution.output
    wait_for_execution_finish = EmrNotebookExecutionSensor(
        task_id="wait_for_execution_finish",
        notebook_execution_id=notebook_execution_id_2,
        poke_interval=5,
    )

    chain(
        # TEST SETUP
        test_context,
        # TEST BODY
        start_execution,
        wait_for_execution_start,
        stop_execution,
        wait_for_execution_stop,
        finish_execution,
        # TEST TEARDOWN
        wait_for_execution_finish,
    )

    from tests_common.test_utils.watcher import watcher

    # This test needs watcher in order to properly mark success/failure
    # when "tearDown" task with trigger rule is part of the DAG
    list(dag.tasks) >> watcher()

from tests_common.test_utils.system_tests import get_test_run  # noqa: E402

# Needed to run the example DAG with pytest (see: tests/system/README.md#run_via_pytest)
test_run = get_test_run(dag)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from __future__ import annotations

from collections.abc import Generator
from unittest import mock

import pytest
from moto import mock_aws

from airflow.providers.amazon.aws.hooks.neptune import NeptuneHook


@pytest.fixture
def neptune_hook() -> Generator[NeptuneHook, None, None]:
    """Returns a NeptuneHook mocked with moto"""
    with mock_aws():
        yield NeptuneHook(aws_conn_id="aws_default")


@pytest.fixture
def neptune_cluster_id(neptune_hook: NeptuneHook) -> str:
    """Returns Neptune cluster ID"""
    resp = neptune_hook.conn.create_db_cluster(
        DBClusterIdentifier="test-cluster",
        Engine="neptune",
    )

    return resp["DBCluster"]["DBClusterIdentifier"]


class TestNeptuneHook:
    def test_get_conn_returns_a_boto3_connection(self):
        hook = NeptuneHook(aws_conn_id="aws_default")
        assert hook.get_conn() is not None

    def test_get_cluster_status(self, neptune_hook: NeptuneHook, neptune_cluster_id):
        assert neptune_hook.get_cluster_status(neptune_cluster_id) is not None

    @mock.patch.object(NeptuneHook, "get_waiter")
    def test_wait_for_cluster_instance_availability(
        self, mock_get_waiter, neptune_hook: NeptuneHook, neptune_cluster_id
    ):
        neptune_hook.wait_for_cluster_instance_availability(neptune_cluster_id)
        mock_get_waiter.assert_called_once_with("db_instance_available")
